@@ -17,7 +17,7 @@
 // |          Bertrand Mansion <bmansion@mamasam.com>                     |
 // +----------------------------------------------------------------------+
 //
-// $Id: group.php,v 1.33 2004/02/08 16:01:28 mansion Exp $
+// $Id: group.php,v 1.36 2005/07/22 14:19:58 avb Exp $
 
 require_once("HTML/QuickForm/element.php");
 
@@ -151,9 +151,7 @@ class HTML_QuickForm_group extends HTML_QuickForm_element
      */
     function setValue($value)
     {
-        if (empty($this->_elements)) {
-            $this->_createElements();
-        }
+        $this->_createElementsIfNotExist();
         foreach (array_keys($this->_elements) as $key) {
             if (!$this->_appendName) {
                 $v = $this->_elements[$key]->_findValue($value);
@@ -163,7 +161,7 @@ class HTML_QuickForm_group extends HTML_QuickForm_element
 
             } else {
                 $elementName = $this->_elements[$key]->getName();
-                $index       = (!empty($elementName)) ? $elementName : $key;
+                $index       = strlen($elementName) ? $elementName : $key;
                 if (is_array($value)) {
                     if (isset($value[$index])) {
                         $this->_elements[$key]->onQuickFormEvent('setGroupValue', $value[$index], $this);
@@ -208,7 +206,7 @@ class HTML_QuickForm_group extends HTML_QuickForm_element
                     if (!is_array($value)) {
                         $value = is_null($value)? array(): array($value);
                     }
-                    if ('' == $elementName) {
+                    if ('' === $elementName) {
                         $value[] = $v;
                     } else {
                         $value[$elementName] = $v;
@@ -233,6 +231,9 @@ class HTML_QuickForm_group extends HTML_QuickForm_element
     function setElements($elements)
     {
         $this->_elements = array_values($elements);
+        if ($this->_flagFrozen) {
+            $this->freeze();
+        }
     } // end func setElements
 
     // }}}
@@ -247,6 +248,7 @@ class HTML_QuickForm_group extends HTML_QuickForm_element
      */
     function &getElements()
     {
+        $this->_createElementsIfNotExist();
         return $this->_elements;
     } // end func getElements
 
@@ -263,9 +265,7 @@ class HTML_QuickForm_group extends HTML_QuickForm_element
      */
     function getGroupType()
     {
-        if (empty($this->_elements)) {
-            $this->_createElements();
-        }
+        $this->_createElementsIfNotExist();
         $prevType = '';
         foreach (array_keys($this->_elements) as $key) {
             $type = $this->_elements[$key]->getType();
@@ -309,9 +309,7 @@ class HTML_QuickForm_group extends HTML_QuickForm_element
      */
     function getElementName($index)
     {
-        if (empty($this->_elements)) {
-            $this->_createElements();
-        }
+        $this->_createElementsIfNotExist();
         $elementName = false;
         if (is_int($index) && isset($this->_elements[$index])) {
             $elementName = $this->_elements[$index]->getName();
@@ -354,10 +352,19 @@ class HTML_QuickForm_group extends HTML_QuickForm_element
      */
     function getFrozenHtml()
     {
-        $tmp = $this->_flagFrozen;
-        $this->_flagFrozen = true;
+        $flags = array();
+        $this->_createElementsIfNotExist();
+        foreach (array_keys($this->_elements) as $key) {
+            if (false === ($flags[$key] = $this->_elements[$key]->isFrozen())) {
+                $this->_elements[$key]->freeze();
+            }
+        }
         $html = $this->toHtml();
-        $this->_flagFrozen = $tmp;
+        foreach (array_keys($this->_elements) as $key) {
+            if (!$flags[$key]) {
+                $this->_elements[$key]->unfreeze();
+            }
+        }
         return $html;
     } //end func getFrozenHtml
 
@@ -378,15 +385,13 @@ class HTML_QuickForm_group extends HTML_QuickForm_element
     {
         switch ($event) {
             case 'updateValue':
-                if (empty($this->_elements)) {
-                    $this->_createElements();
-                }
+                $this->_createElementsIfNotExist();
                 foreach (array_keys($this->_elements) as $key) {
                     if ($this->_appendName) {
                         $elementName = $this->_elements[$key]->getName();
                         if (is_null($elementName)) {
                             $this->_elements[$key]->setName($this->getName());
-                        } elseif ('' == $elementName) {
+                        } elseif ('' === $elementName) {
                             $this->_elements[$key]->setName($this->getName() . '[' . $key . ']');
                         } else {
                             $this->_elements[$key]->setName($this->getName() . '[' . $elementName . ']');
@@ -419,9 +424,7 @@ class HTML_QuickForm_group extends HTML_QuickForm_element
     */
     function accept(&$renderer, $required = false, $error = null)
     {
-        if (empty($this->_elements)) {
-            $this->_createElements();
-        }
+        $this->_createElementsIfNotExist();
         $renderer->startGroup($this, $required, $error);
         $name = $this->getName();
         foreach (array_keys($this->_elements) as $key) {
@@ -436,10 +439,7 @@ class HTML_QuickForm_group extends HTML_QuickForm_element
                 }
             }
 
-            if ($this->_flagFrozen) {
-                $element->freeze();
-            }
-            $required = !$this->_flagFrozen && in_array($element->getName(), $this->_required);
+            $required = !$element->isFrozen() && in_array($element->getName(), $this->_required);
 
             $element->accept($renderer, $required);
 
@@ -466,7 +466,7 @@ class HTML_QuickForm_group extends HTML_QuickForm_element
             if ($this->_appendName) {
                 if (is_null($elementName)) {
                     $this->_elements[$key]->setName($this->getName());
-                } elseif ('' == $elementName) {
+                } elseif ('' === $elementName) {
                     $this->_elements[$key]->setName($this->getName() . '[' . $key . ']');
                 } else {
                     $this->_elements[$key]->setName($this->getName() . '[' . $elementName . ']');
@@ -488,7 +488,7 @@ class HTML_QuickForm_group extends HTML_QuickForm_element
                     // just like getValue(), but should work OK every time here
                     if (is_null($elementName)) {
                         $value = $v;
-                    } elseif ('' == $elementName) {
+                    } elseif ('' === $elementName) {
                         $value[] = $v;
                     } else {
                         $value[$elementName] = $v;
@@ -519,6 +519,61 @@ class HTML_QuickForm_group extends HTML_QuickForm_element
         // abstract
     }
 
+    // }}}
+    // {{{ _createElementsIfNotExist()
+
+   /**
+    * A wrapper around _createElements()
+    *
+    * This method calls _createElements() if the group's _elements array
+    * is empty. It also performs some updates, e.g. freezes the created
+    * elements if the group is already frozen.
+    *
+    * @access private
+    */
+    function _createElementsIfNotExist()
+    {
+        if (empty($this->_elements)) {
+            $this->_createElements();
+            if ($this->_flagFrozen) {
+                $this->freeze();
+            }
+        }
+    }
+
+    // }}}
+    // {{{ freeze()
+
+    function freeze()
+    {
+        parent::freeze();
+        foreach (array_keys($this->_elements) as $key) {
+            $this->_elements[$key]->freeze();
+        }
+    }
+
+    // }}}
+    // {{{ unfreeze()
+
+    function unfreeze()
+    {
+        parent::unfreeze();
+        foreach (array_keys($this->_elements) as $key) {
+            $this->_elements[$key]->unfreeze();
+        }
+    }
+
+    // }}}
+    // {{{ setPersistantFreeze()
+
+    function setPersistantFreeze($persistant = false)
+    {
+        parent::setPersistantFreeze($persistant);
+        foreach (array_keys($this->_elements) as $key) {
+            $this->_elements[$key]->setPersistantFreeze($persistant);
+        }
+    }
+
     // }}}
 } //end class HTML_QuickForm_group
 ?>
\ No newline at end of file
